------------------------------------------------------------------------------------------------
-- Starcraft 2 Model Objects version 0.02
-- by NiNtoxicated (madyavic@gmail.com)
-- 3ds max objects for use with Starcraft 2 model files (.M3)
-- Created for 3ds max 2010, but should be backwards compatible
--
-- Objects:
-- sc2material - Material plugin for setting material and layer properties
-- sc2bitmap - Bitmap plugin with extra map options
--
------------------------------------------------------------------------------------------------
--
-- Version History:
-- 0.02 - May 17th 2010
-- sc2bitmap definition updated
-- Integrated animation UI
--
-- 0,01 - May 15th 2010
-- Initial objects script version
-- sc2material defined
-- sc2bitmap defined
--
------------------------------------------------------------------------------------------------

-- ***************
--  M3 OBJECTS
-- *************** 
plugin Material sc2material
name:"Starcraft 2" 
classID:#(0x1ccf0cb1, 0x1e0c2826)
extends:Standard replaceUI:true version:1
( 
	parameters matsett rollout:matsettui
	(
		blendMode type:#integer default:1 ui:ddlBlendMode
		priority type:#integer default:0 ui:spnPriority
		cutoutThresh type:#integer default:0 ui:spnCutoutThresh
		specVal type:#float default:0 ui:spnSpec
		specMult type:#float default:1 ui:spnSpecMult
		emisMult type:#float default:1 ui:spnEmisMult
		layerBlendType type:#integer default:3 ui:ddlLayerBlend
		emissiveBlendType type:#integer default:3 ui:ddlEmissiveBlend
			
		on specVal set val do delegate.specularLevel = val
	)
	
	parameters flags rollout:flagsui
	(
		Unfogged type:#boolean ui:chkUnfogged
		TwoSided type:#boolean ui:chkTwoSided
		Unshaded type:#boolean ui:chkUnshaded
		NoHitTest type:#boolean ui:chkNoHitTest
		DepthPrepass type:#boolean ui:chkDepthPrepass
		UseTerrainHDR type:#boolean ui:chkTerrainHDR
		SplatUVfix type:#boolean ui:chkSplatUV
		SoftBlending type:#boolean ui:chkSoftBlend
		NoShadowsCast type:#boolean ui:chkNoShadowsCast
		NoShadowsReceived type:#boolean ui:chkNoShadowsRec
	)
	
	parameters layers rollout:layersui
	(
		diffuseMap type:#textureMap ui:mbDiffuse
		decalMap type:#textureMap ui:mbDecal
		specularMap type:#textureMap ui:mbSpec
		selfIllumMap type:#textureMap ui:mbEmis
		envioMap type:#textureMap ui:mbEnvio
		envioMaskMap type:#textureMap ui:mbEnvioMask
		alphaMaskMap type:#textureMap ui:mbAlphaMask
		bumpMap type:#textureMap ui:mbNorm
		heightMap type:#textureMap ui:mbHeight

		-- Set Maps
		on diffuseMap set val do delegate.diffuseMap = val
		on specularMap set val do delegate.specularMap = val
		on selfIllumMap set val do delegate.selfIllumMap = val
		on bumpMap set val do delegate.bumpMap = val
		--on envioMap set val do delegate.ReflectionMap = val
	)

	rollout matsettui "Material Settings"
	(
		group "Blend Types"
		(
			dropdownlist ddlBlendMode "Blend Mode:" items:#("Opaque", "Alpha Blend", "Add", "Alpha Add",  "Mod", "Mod 2x") across:2 align:#left width:85 across:2
			dropdownlist ddlLayerBlend "Layer Blend Type:" items:#("Mod", "Mod 2x", "Add", "Blend", "Team Colour Emissive Add", "Team Colour Diffuse Add") selection:3 align:#left width:120 
			dropdownlist ddlEmissiveBlend "Emissive Blend Type:" items:#("Mod", "Mod 2x", "Add", "Blend", "Team Colour Emissive Add", "Team Colour Diffuse Add") selection:3 align:#left width:120 
		)

		group "Properties"
		(
			spinner spnPriority "Priority: " fieldwidth:40 range:[0,1e9,0] type:#integer align:#left across:2
			spinner spnCutoutThresh "Cutout Threshold: " fieldwidth:40 range:[0,1e9,0] type:#integer scale:1 
			spinner spnSpec "Specularity: " fieldwidth:45 range:[0,1e9,0] scale:1 align:#left across:2
			spinner spnEmisMult "HDR Emissive Multiplier: " fieldwidth:45 range:[0,1e9,1] scale:1 align:#right
			spinner spnSpecMult "HDR Specular Multiplier: " fieldwidth:45 range:[0,1e9,1] scale:1 align:#right 
		)
	)
	
	rollout flagsui "Flags"
	(
		group "Flags"
		(
			checkbox chkUnfogged "Unfogged" align:#left across:3
			checkbox chkTwoSided "Two Sided"
			checkbox chkUnshaded "Unshaded"
			
			checkbox chkNoHitTest "No Hit Test" align:#left across:3
			checkbox chkDepthPrepass "Depth Pre-pass"
			checkbox chkTerrainHDr "Use Terrain HDR" 
			
			checkbox chkSplatUV "Splat UV Fix" align:#left across:3
			checkbox chkSoftBlend "Soft Blending"			
			checkbox chkNoShadowsCast "No Shadows Cast"
			
			checkbox chkNoShadowsRec "No Shadows Received"
		)
	)
	
	rollout layersui "Layers"
	(
		group "Map Layers"
		(
			label lblDiffuse "Diffuse Map: " across:2 align:#left
			mapbutton mbDiffuse "None" tooltip:"Select Diffuse Map" offset:[-45,0] width:175
			label lblDecal "Decal Map: " across:2 align:#left
			mapbutton mbDecal "None" tooltip:"Select Decal Map" offset:[-45,0] width:175
			label lblSpec "Specular Map: " across:2 align:#left
			mapbutton mbSpec "None" tooltip:"Select Specular Map" offset:[-45,0] width:175
			label lblEmis "Emissive Map: " across:2 align:#left
			mapbutton mbEmis "None" tooltip:"Select Emissive Map" offset:[-45,0] width:175
			label lblEnvio "Envio Map: " across:2 align:#left
			mapbutton mbEnvio "None" tooltip:"Select Envio Map" offset:[-45,0] width:175
			label lblEnvioMask "Envio Mask Map: " across:2 align:#left
			mapbutton mbEnvioMask "None" tooltip:"Select Envio Mask Map" offset:[-45,0] width:175
			label lblAlphaMask "Alpha Mask Map: " across:2 align:#left
			mapbutton mbAlphaMask "None" tooltip:"Select Alpha Mask Map" offset:[-45,0] width:175
			label lblNorm "Normal Map: " across:2 align:#left
			mapbutton mbNorm "None" tooltip:"Select Normal Map" offset:[-45,0] width:175
			label lblHeight "Height Map: " across:2 align:#left
			mapbutton mbHeight "None" tooltip:"Select Height Map" offset:[-45,0] width:175
		)
	)

	on create do
	(
		-- setup initial material
		delegate.bumpMapAmount = 100
	)
)

plugin textureMap sc2bitmap
name:"Starcraft 2 Bitmap"
classID:#(0x48f387a2, 0x6950179f)
extends:Bitmaptexture replaceUI:true version:2
(
	fn assign_bitmap uifname =
	(
		local bmap = SelectBitmap()
		if (bmap != undefined) then
		(
			-- update bitmap and UI
			delegate.bitmap = bmap
			uifname.text = bmap.filename
		)
		else
		(
			-- else leave it as it was
			bmap = this.scbitmap
		)
		return bmap
	)
	
	parameters main rollout:params
	(
		scbitmap type:#bitmap
		TeamColour type:#boolean ui:chkTeamColour
		TexWrap type:#boolean default:true ui:chkTexWrap
		TexAlphaOnly type:#boolean ui:chkTexAlphaOnly
		
		on scbitmap set val do
		(
			if (scbitmap != undefined) then
			(
				delegate.bitmap = scbitmap
			)
		)
	)--end params

	rollout params "Starcraft 2"
	(
		group "Settings"
		(
			label	lblBitmap "Bitmap:" align:#left across:2
			button btnBitmap "None" offset:[-55,0] width:260
			checkbox chkTeamColour "Team Colour" align:#left across:2
			checkbox chkTexWrap "Texture Wrap" checked:true
			checkbox chkTexAlphaOnly "Use Texture Alpha Only" checked:false
		)
		
		on params open do
		(
			if (scbitmap != undefined) then
			(
				btnBitmap.text = scbitmap.filename
			)
			else btnBitmap.text = "None"
			if (chkTexAlphaOnly.checked == true) then
			(
				chkTeamColour.checked = false
				chkTeamColour.enabled = false
			)
		)
		on btnBitmap pressed do
		(
			local map = assign_bitmap btnBitmap
			if (map != undefined) then
			(
				this.scbitmap = map
			)
		)
		on chkTexAlphaOnly changed state do
		(
			if (state == false) then
			(
				chkTeamColour.enabled = true
			)
			else
			(
				chkTeamColour.checked = false
				chkTeamColour.enabled = false
			)
		)
	)--end rollout
)--end plugin

-- ********************
--  M3 UI ELEMENTS
-- ********************
-- Animation UI dependents
struct sc2anim
(
	name, cstart, cend
)

fn Get_Scene_Data obj &ind  =
(
	ind+=1 
	return getAppData obj (ind-1)
)

fn Get_Scene_Section obj &ind  =
(
	ss = sc2anim()
	ss.name 	= Get_Scene_Data obj &ind
	ss.cstart 	= Get_Scene_Data obj &ind
	ss.cend 		= Get_Scene_Data obj &ind 
	return ss
)

-- Animation UI
utility sc2animUI "M3 - Sequences"
(
	local animList = #("None")
	local aobj = $_M3I_Anim_Data
	local adata = #()
	
	dropdownlist	dlAnim "Animation:" items:#("None")
	edittext		edtStart "Time Start:" enabled:false
	edittext		edtEnd "Time End: " enabled:false
	
	fn Selected sel =
	(
		local ad = adata[sel-1]
		animationRange = interval (ad.cstart as integer) (ad.cend as integer)
		sliderTime = (ad.cstart as integer)
		edtStart.text = ad.cstart
		edtEnd.text = ad.cend
	)
	
	fn NoneSelected =
	(
		if (adata.count > 0) then 
		(
			animationRange = interval 0 (adata[adata.count].cend as integer)
			edtEnd.text = adata[adata.count].cend
		)
		else 
		(
			animationRange = interval 0 100
			edtEnd.text = "100"
		)
		edtStart.text = "0"
		edtStart.enabled = false
		edtEnd.enabled = false
	)
	
	fn List_Repop  =
	(
		animList = dlAnim.items
		animList[adata.count+1] = 0
		animList[1] = "None"
		for i = 1 to adata.count do
		(
			--echo (anim_data[i].name)
			animList[i+1] = adata[i].name
		)
		
		rsel = random 1 adata.count
		
		dlAnim.items = animList
		dlAnim.selection = rsel + 1
		Selected dlAnim.selection
	)
	
	fn List_Pop =
	(
		if (aobj != undefined) do
		(	
			--reset vars
			adata = #()
			
			local t = 1
			local animCount = (Get_Scene_Data aobj &t) as integer
			adata[animCount] = 0
			for h = 1 to animCount do
			(
				adata[h] = Get_Scene_Section aobj &t
			)
			
			List_Repop()
		)			
	)
	
	on sc2animUI open do
	(
		List_Pop()
	)
	on dlAnim selected i do
	(
		if (adata.count > 0) then
		(
			if (i == 1) then
			(
				NoneSelected()
			)
			else
			(
				Selected i
			)
		)
	)
)


